"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeedbackDialog = void 0;
const preact_1 = require("preact");
const hooks_1 = require("preact/hooks");
const defaultTranslations_1 = require("./config/defaultTranslations");
const useTimer_1 = require("./hooks/useTimer");
const Close_1 = require("./icons/Close");
const floating_ui_preact_dom_1 = require("./packages/floating-ui-preact-dom");
const constants_1 = require("./constants");
const FeedbackForm_1 = require("./FeedbackForm");
const index_css_inline_1 = __importDefault(require("./index.css?inline"));
const RadialProgress_1 = require("./RadialProgress");
const INACTIVE_DURATION_MS = 20 * 1000;
const SUCCESS_DURATION_MS = 3 * 1000;
const FeedbackDialog = ({ key, title = defaultTranslations_1.DEFAULT_TRANSLATIONS.DefaultQuestionLabel, position, translations = defaultTranslations_1.DEFAULT_TRANSLATIONS, openWithCommentVisible = false, onClose, onDismiss, onSubmit, onScoreSubmit, }) => {
    var _a, _b, _c;
    const arrowRef = (0, hooks_1.useRef)(null);
    const anchor = position.type === "POPOVER" ? position.anchor : null;
    const { refs, floatingStyles, middlewareData, placement: actualPlacement, } = (0, floating_ui_preact_dom_1.useFloating)({
        elements: {
            reference: anchor,
        },
        transform: false,
        whileElementsMounted: floating_ui_preact_dom_1.autoUpdate,
        middleware: [
            (0, floating_ui_preact_dom_1.shift)(),
            (0, floating_ui_preact_dom_1.offset)(8),
            (0, floating_ui_preact_dom_1.arrow)({
                element: arrowRef,
            }),
        ],
    });
    let unanchoredPosition = {};
    if (position.type === "DIALOG") {
        const offsetY = parseOffset((_a = position.offset) === null || _a === void 0 ? void 0 : _a.y);
        const offsetX = parseOffset((_b = position.offset) === null || _b === void 0 ? void 0 : _b.x);
        switch (position.placement) {
            case "top-left":
                unanchoredPosition = {
                    top: offsetY,
                    left: offsetX,
                };
                break;
            case "top-right":
                unanchoredPosition = {
                    top: offsetY,
                    right: offsetX,
                };
                break;
            case "bottom-left":
                unanchoredPosition = {
                    bottom: offsetY,
                    left: offsetX,
                };
                break;
            case "bottom-right":
                unanchoredPosition = {
                    bottom: offsetY,
                    right: offsetX,
                };
                break;
        }
    }
    const { x: arrowX, y: arrowY } = (_c = middlewareData.arrow) !== null && _c !== void 0 ? _c : {};
    const staticSide = {
        top: "bottom",
        right: "left",
        bottom: "top",
        left: "right",
    }[actualPlacement.split("-")[0]] || "bottom";
    const arrowStyles = {
        left: arrowX != null ? `${arrowX}px` : "",
        top: arrowY != null ? `${arrowY}px` : "",
        right: "",
        bottom: "",
        [staticSide]: "-4px",
    };
    const close = (0, hooks_1.useCallback)(() => {
        const dialog = refs.floating.current;
        dialog === null || dialog === void 0 ? void 0 : dialog.close();
        autoClose.stop();
        onClose === null || onClose === void 0 ? void 0 : onClose();
    }, [onClose]);
    const dismiss = (0, hooks_1.useCallback)(() => {
        close();
        onDismiss === null || onDismiss === void 0 ? void 0 : onDismiss();
    }, [close, onDismiss]);
    const [feedbackId, setFeedbackId] = (0, hooks_1.useState)(undefined);
    const [scoreState, setScoreState] = (0, hooks_1.useState)("idle");
    const submit = (0, hooks_1.useCallback)((data) => __awaiter(void 0, void 0, void 0, function* () {
        yield onSubmit(Object.assign(Object.assign({}, data), { feedbackId }));
        autoClose.startWithDuration(SUCCESS_DURATION_MS);
    }), [feedbackId, onSubmit]);
    const submitScore = (0, hooks_1.useCallback)((data) => __awaiter(void 0, void 0, void 0, function* () {
        if (onScoreSubmit !== undefined) {
            setScoreState("submitting");
            const res = yield onScoreSubmit(Object.assign(Object.assign({}, data), { feedbackId }));
            setFeedbackId(res.feedbackId);
            setScoreState("submitted");
        }
    }), [feedbackId, onSubmit]);
    const autoClose = (0, useTimer_1.useTimer)({
        enabled: position.type === "DIALOG",
        initialDuration: INACTIVE_DURATION_MS,
        onEnd: close,
    });
    (0, hooks_1.useEffect)(() => {
        // Only enable 'quick dismiss' for popovers
        if (position.type === "MODAL" || position.type === "DIALOG")
            return;
        const escapeHandler = (e) => {
            if (e.key == "Escape") {
                dismiss();
            }
        };
        const clickOutsideHandler = (e) => {
            if (!(e.target instanceof Element) ||
                !e.target.closest(`#${constants_1.feedbackContainerId}`)) {
                dismiss();
            }
        };
        const observer = new MutationObserver((mutations) => {
            if (position.anchor === null)
                return;
            mutations.forEach((mutation) => {
                const removedNodes = Array.from(mutation.removedNodes);
                const hasBeenRemoved = removedNodes.some((node) => {
                    return node === position.anchor || node.contains(position.anchor);
                });
                if (hasBeenRemoved) {
                    close();
                }
            });
        });
        window.addEventListener("mousedown", clickOutsideHandler);
        window.addEventListener("keydown", escapeHandler);
        observer.observe(document.body, {
            subtree: true,
            childList: true,
        });
        return () => {
            window.removeEventListener("mousedown", clickOutsideHandler);
            window.removeEventListener("keydown", escapeHandler);
            observer.disconnect();
        };
    }, [position.type, close]);
    return ((0, preact_1.h)(preact_1.Fragment, null,
        (0, preact_1.h)("style", { dangerouslySetInnerHTML: { __html: index_css_inline_1.default } }),
        (0, preact_1.h)("dialog", { ref: refs.setFloating, class: [
                "dialog",
                position.type === "MODAL"
                    ? "modal"
                    : position.type === "POPOVER"
                        ? "anchored"
                        : `unanchored unanchored-${position.placement}`,
                actualPlacement,
            ].join(" "), style: anchor ? floatingStyles : unanchoredPosition },
            (0, preact_1.h)(FeedbackForm_1.FeedbackForm, { t: Object.assign(Object.assign({}, defaultTranslations_1.DEFAULT_TRANSLATIONS), translations), key: key, question: title, openWithCommentVisible: openWithCommentVisible, onSubmit: submit, onScoreSubmit: submitScore, scoreState: scoreState, onInteraction: autoClose.stop }),
            (0, preact_1.h)("button", { onClick: dismiss, class: "close" },
                !autoClose.stopped && autoClose.elapsedFraction > 0 && ((0, preact_1.h)(RadialProgress_1.RadialProgress, { diameter: 28, progress: 1.0 - autoClose.elapsedFraction })),
                (0, preact_1.h)(Close_1.Close, null)),
            anchor && ((0, preact_1.h)("div", { ref: arrowRef, class: ["arrow", actualPlacement].join(" "), style: arrowStyles })))));
};
exports.FeedbackDialog = FeedbackDialog;
function parseOffset(offsetInput) {
    if (offsetInput === undefined)
        return "1rem";
    if (typeof offsetInput === "number")
        return offsetInput + "px";
    return offsetInput;
}
//# sourceMappingURL=FeedbackDialog.js.map