"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FeedbackForm = void 0;
const preact_1 = require("preact");
const hooks_1 = require("preact/hooks");
const Check_1 = require("./icons/Check");
const CheckCircle_1 = require("./icons/CheckCircle");
const Button_1 = require("./Button");
const Plug_1 = require("./Plug");
const StarRating_1 = require("./StarRating");
const ANIMATION_SPEED = 400;
function getFeedbackDataFromForm(el) {
    var _a, _b;
    const formData = new FormData(el);
    return {
        score: Number((_a = formData.get("score")) === null || _a === void 0 ? void 0 : _a.toString()),
        comment: (((_b = formData.get("comment")) === null || _b === void 0 ? void 0 : _b.toString()) || "").trim(),
    };
}
const FeedbackForm = ({ question, scoreState, openWithCommentVisible, onInteraction, onSubmit, onScoreSubmit, t, }) => {
    const [hasRating, setHasRating] = (0, hooks_1.useState)(false);
    const [status, setStatus] = (0, hooks_1.useState)("idle");
    const [error, setError] = (0, hooks_1.useState)();
    const [showForm, setShowForm] = (0, hooks_1.useState)(true);
    const handleSubmit = (e) => __awaiter(void 0, void 0, void 0, function* () {
        e.preventDefault();
        const data = Object.assign(Object.assign({}, getFeedbackDataFromForm(e.target)), { question });
        if (!data.score)
            return;
        setError("");
        try {
            setStatus("submitting");
            yield onSubmit(data);
            setStatus("submitted");
        }
        catch (err) {
            setStatus("idle");
            if (err instanceof Error) {
                setError(err.message);
            }
            else if (typeof err === "string") {
                setError(err);
            }
            else {
                setError("Couldn't submit feedback. Please try again.");
            }
        }
    });
    const containerRef = (0, hooks_1.useRef)(null);
    const formRef = (0, hooks_1.useRef)(null);
    const headerRef = (0, hooks_1.useRef)(null);
    const expandedContentRef = (0, hooks_1.useRef)(null);
    const submittedRef = (0, hooks_1.useRef)(null);
    const transitionToDefault = (0, hooks_1.useCallback)(() => {
        if (containerRef.current === null)
            return;
        if (headerRef.current === null)
            return;
        if (expandedContentRef.current === null)
            return;
        containerRef.current.style.maxHeight =
            headerRef.current.clientHeight + "px";
        expandedContentRef.current.style.position = "absolute";
        expandedContentRef.current.style.opacity = "0";
        expandedContentRef.current.style.pointerEvents = "none";
    }, [containerRef, headerRef, expandedContentRef]);
    const transitionToExpanded = (0, hooks_1.useCallback)(() => {
        if (containerRef.current === null)
            return;
        if (headerRef.current === null)
            return;
        if (expandedContentRef.current === null)
            return;
        containerRef.current.style.maxHeight =
            headerRef.current.clientHeight + // Header height
                expandedContentRef.current.clientHeight + // Comment + Button Height
                10 + // Gap height
                "px";
        expandedContentRef.current.style.position = "relative";
        expandedContentRef.current.style.opacity = "1";
        expandedContentRef.current.style.pointerEvents = "all";
    }, [containerRef, headerRef, expandedContentRef]);
    const transitionToSuccess = (0, hooks_1.useCallback)(() => {
        if (containerRef.current === null)
            return;
        if (formRef.current === null)
            return;
        if (submittedRef.current === null)
            return;
        formRef.current.style.opacity = "0";
        formRef.current.style.pointerEvents = "none";
        containerRef.current.style.maxHeight =
            submittedRef.current.clientHeight + "px";
        // Fade in "submitted" step once container has resized
        setTimeout(() => {
            submittedRef.current.style.position = "relative";
            submittedRef.current.style.opacity = "1";
            submittedRef.current.style.pointerEvents = "all";
            setShowForm(false);
        }, ANIMATION_SPEED + 10);
    }, [formRef, containerRef, submittedRef]);
    (0, hooks_1.useEffect)(() => {
        if (status === "submitted") {
            transitionToSuccess();
        }
        else if (openWithCommentVisible || hasRating) {
            transitionToExpanded();
        }
        else {
            transitionToDefault();
        }
    }, [
        transitionToDefault,
        transitionToExpanded,
        transitionToSuccess,
        openWithCommentVisible,
        hasRating,
        status,
    ]);
    return ((0, preact_1.h)("div", { ref: containerRef, class: "container" },
        (0, preact_1.h)("div", { ref: submittedRef, class: "submitted" },
            (0, preact_1.h)("div", { className: "submitted-check" },
                (0, preact_1.h)(CheckCircle_1.CheckCircle, { width: 24, height: 24 })),
            (0, preact_1.h)("p", { className: "text" }, t.SuccessMessage),
            (0, preact_1.h)(Plug_1.Plug, null)),
        showForm && ((0, preact_1.h)("form", { ref: formRef, onSubmit: handleSubmit, method: "dialog", class: "form", onFocus: onInteraction, onFocusCapture: onInteraction, onClick: onInteraction, style: { opacity: 1 } },
            (0, preact_1.h)("div", { ref: headerRef, role: "group", class: "form-control", "aria-labelledby": "bucket-feedback-score-label" },
                (0, preact_1.h)("div", { id: "bucket-feedback-score-label", class: "title" }, question),
                (0, preact_1.h)(StarRating_1.StarRating, { t: t, name: "score", onChange: (e) => __awaiter(void 0, void 0, void 0, function* () {
                        setHasRating(true);
                        yield onScoreSubmit({
                            question,
                            score: Number(e.currentTarget.value),
                        });
                    }) }),
                (0, preact_1.h)(ScoreStatus, { t: t, scoreState: scoreState })),
            (0, preact_1.h)("div", { ref: expandedContentRef, class: "form-expanded-content" },
                (0, preact_1.h)("div", { class: "form-control" },
                    (0, preact_1.h)("textarea", { id: "bucket-feedback-comment-label", class: "textarea", name: "comment", placeholder: t.QuestionPlaceholder, rows: 4 })),
                error && (0, preact_1.h)("p", { class: "error" }, error),
                (0, preact_1.h)(Button_1.Button, { type: "submit", disabled: !hasRating ||
                        status === "submitting" ||
                        scoreState === "submitting" }, t.SendButton),
                (0, preact_1.h)(Plug_1.Plug, null))))));
};
exports.FeedbackForm = FeedbackForm;
const ScoreStatus = ({ t, scoreState }) => {
    // Keep track of whether we can show a loading indication - only if 400ms have
    // elapsed without the score request finishing.
    const [loadingTimeElapsed, setLoadingTimeElapsed] = (0, hooks_1.useState)(false);
    // Keep track of whether we can fall back to the idle/loading states - once
    // it's been submit once it won't, to prevent flashing.
    const [hasBeenSubmitted, setHasBeenSubmitted] = (0, hooks_1.useState)(false);
    (0, hooks_1.useEffect)(() => {
        if (scoreState === "idle") {
            setLoadingTimeElapsed(false);
            return;
        }
        if (scoreState === "submitted") {
            setLoadingTimeElapsed(false);
            setHasBeenSubmitted(true);
            return;
        }
        const timer = setTimeout(() => {
            setLoadingTimeElapsed(true);
        }, 400);
        return () => clearTimeout(timer);
    }, [scoreState]);
    const showIdle = scoreState === "idle" ||
        (scoreState === "submitting" && !hasBeenSubmitted && !loadingTimeElapsed);
    const showLoading = scoreState !== "submitted" && !hasBeenSubmitted && loadingTimeElapsed;
    const showSubmitted = scoreState === "submitted" || hasBeenSubmitted;
    return ((0, preact_1.h)("div", { className: "score-status-container" },
        (0, preact_1.h)("span", { className: "score-status", style: { opacity: showIdle ? 1 : 0 } }, t.ScoreStatusDescription),
        (0, preact_1.h)("div", { className: "score-status", style: { opacity: showLoading ? 1 : 0 } }, t.ScoreStatusLoading),
        (0, preact_1.h)("span", { className: "score-status", style: { opacity: showSubmitted ? 1 : 0 } },
            (0, preact_1.h)(Check_1.Check, { width: 14, height: 14, style: { marginRight: 3 } }),
            " ",
            t.ScoreStatusReceived)));
};
//# sourceMappingURL=FeedbackForm.js.map