"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StarRating = void 0;
const preact_1 = require("preact");
const hooks_1 = require("preact/hooks");
const Dissatisfied_1 = require("./icons/Dissatisfied");
const Neutral_1 = require("./icons/Neutral");
const Satisfied_1 = require("./icons/Satisfied");
const VeryDissatisfied_1 = require("./icons/VeryDissatisfied");
const VerySatisfied_1 = require("./icons/VerySatisfied");
const floating_ui_preact_dom_1 = require("./packages/floating-ui-preact-dom");
const scores = [
    {
        color: "var(--bucket-feedback-dialog-rating-1-color, #dd6b20)",
        bg: "var(--bucket-feedback-dialog-rating-1-background-color, #fbd38d)",
        icon: (0, preact_1.h)(VeryDissatisfied_1.VeryDissatisfied, null),
        getLabel: (t) => t.ScoreVeryDissatisfiedLabel,
        value: 1,
    },
    {
        color: "var(--bucket-feedback-dialog-rating-2-color, #ed8936)",
        bg: "var(--bucket-feedback-dialog-rating-2-background-color, #feebc8)",
        icon: (0, preact_1.h)(Dissatisfied_1.Dissatisfied, null),
        getLabel: (t) => t.ScoreDissatisfiedLabel,
        value: 2,
    },
    {
        color: "var(--bucket-feedback-dialog-rating-3-color, #787c91)",
        bg: "var(--bucket-feedback-dialog-rating-3-background-color, #e9e9ed)",
        icon: (0, preact_1.h)(Neutral_1.Neutral, null),
        getLabel: (t) => t.ScoreNeutralLabel,
        value: 3,
    },
    {
        color: "var(--bucket-feedback-dialog-rating-4-color, #48bb78)",
        bg: "var(--bucket-feedback-dialog-rating-4-background-color, #c6f6d5)",
        icon: (0, preact_1.h)(Satisfied_1.Satisfied, null),
        getLabel: (t) => t.ScoreSatisfiedLabel,
        value: 4,
    },
    {
        color: "var(--bucket-feedback-dialog-rating-5-color, #38a169)",
        bg: "var(--bucket-feedback-dialog-rating-5-background-color, #9ae6b4)",
        icon: (0, preact_1.h)(VerySatisfied_1.VerySatisfied, null),
        getLabel: (t) => t.ScoreVerySatisfiedLabel,
        value: 5,
    },
];
const StarRating = ({ t, name, selectedValue, onChange, }) => {
    return ((0, preact_1.h)("div", { class: "star-rating" },
        (0, preact_1.h)("style", null, scores.map(({ bg, color }, index) => `
            .star-rating-icons > input:nth-of-type(${index + 1}):checked + .button {
              border-color: ${color};
            }

            .star-rating-icons > input:nth-of-type(${index + 1}):checked + .button > div {
              background-color: ${bg};
            }

            .star-rating-icons > input:nth-of-type(${index + 1}):checked ~ input:nth-of-type(${index + 2}) + .button {
              border-left-color: ${color};
            }
          `)),
        (0, preact_1.h)("div", { class: "star-rating-icons" }, scores.map((score) => ((0, preact_1.h)(Score, { isSelected: score.value === selectedValue, name: name, onChange: onChange, score: score, t: t }))))));
};
exports.StarRating = StarRating;
const Score = ({ isSelected, name, onChange, score, t, }) => {
    var _a, _b;
    const arrowRef = (0, hooks_1.useRef)(null);
    const { refs, floatingStyles, middlewareData } = (0, floating_ui_preact_dom_1.useFloating)({
        placement: "top",
        middleware: [
            (0, floating_ui_preact_dom_1.offset)(4),
            (0, floating_ui_preact_dom_1.arrow)({
                element: arrowRef,
            }),
        ],
    });
    return ((0, preact_1.h)(preact_1.Fragment, null,
        (0, preact_1.h)("input", { id: `bucket-feedback-score-${score.value}`, type: "radio", name: name, value: score.value, defaultChecked: isSelected, onChange: onChange }),
        (0, preact_1.h)("label", { ref: refs.setReference, for: `bucket-feedback-score-${score.value}`, class: "button", style: { color: score.color }, "aria-label": score.getLabel(t) },
            (0, preact_1.h)("div", { style: {
                    position: "absolute",
                    top: 0,
                    left: 0,
                    width: "100%",
                    height: "100%",
                    opacity: 0.2, // TODO: fix overflow
                    zIndex: 1,
                } }),
            (0, preact_1.h)("span", { style: { zIndex: 2, display: "flex", alignItems: "center" } }, score.icon)),
        (0, preact_1.h)("div", { ref: refs.setFloating, class: "button-tooltip", style: floatingStyles },
            score.getLabel(t),
            (0, preact_1.h)("div", { ref: arrowRef, class: "button-tooltip-arrow", style: {
                    left: ((_a = middlewareData.arrow) === null || _a === void 0 ? void 0 : _a.x) != null
                        ? `${middlewareData.arrow.x}px`
                        : "",
                    top: ((_b = middlewareData.arrow) === null || _b === void 0 ? void 0 : _b.y) != null
                        ? `${middlewareData.arrow.y}px`
                        : "",
                } }))));
};
//# sourceMappingURL=StarRating.js.map