"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTimer = void 0;
const hooks_1 = require("preact/hooks");
const useTimer = ({ initialDuration, enabled, onEnd, }) => {
    const [stopped, setStopped] = (0, hooks_1.useState)(!enabled);
    const [duration, setDuration] = (0, hooks_1.useState)(initialDuration);
    const [startTime, setStartTime] = (0, hooks_1.useState)(Date.now());
    const [currentTime, setCurrentTime] = (0, hooks_1.useState)(Date.now());
    (0, hooks_1.useEffect)(() => {
        if (stopped)
            return;
        const t = setInterval(() => {
            setCurrentTime(Date.now());
            if (Date.now() >= startTime + duration) {
                clearTimeout(t);
                setStopped(true);
                onEnd();
            }
        }, 25);
        return () => {
            clearTimeout(t);
        };
    }, [stopped]);
    const stop = (0, hooks_1.useCallback)(() => {
        setStopped(true);
    }, []);
    const startWithDuration = (0, hooks_1.useCallback)((nextDuration) => {
        setStartTime(Date.now());
        setDuration(nextDuration);
        setStopped(false);
    }, []);
    const endTime = startTime + duration;
    const elapsedMs = stopped ? 0 : currentTime - startTime;
    const elapsedFraction = elapsedMs / duration;
    return {
        duration,
        elapsedFraction,
        startTime,
        endTime,
        stopped,
        startWithDuration,
        stop,
    };
};
exports.useTimer = useTimer;
//# sourceMappingURL=useTimer.js.map