"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.openFeedbackForm = void 0;
const preact_1 = require("preact");
const constants_1 = require("./constants");
const FeedbackDialog_1 = require("./FeedbackDialog");
const DEFAULT_POSITION = {
    type: "DIALOG",
    placement: "bottom-right",
};
function stopPropagation(e) {
    e.stopPropagation();
}
function attachDialogContainer() {
    let container = document.querySelector(`#${constants_1.feedbackContainerId}`);
    if (!container) {
        container = document.createElement("div");
        container.attachShadow({ mode: "open" });
        container.style.all = "initial";
        container.id = constants_1.feedbackContainerId;
        document.body.appendChild(container);
        for (const event of constants_1.propagatedEvents) {
            container.addEventListener(event, stopPropagation);
        }
    }
    return container.shadowRoot;
}
function openFeedbackForm(options) {
    const shadowRoot = attachDialogContainer();
    const position = options.position || DEFAULT_POSITION;
    if (position.type === "POPOVER") {
        if (!position.anchor) {
            console.warn("[Bucket]", "Unable to open popover. Anchor must be a defined DOM-element");
            return;
        }
        if (!document.body.contains(position.anchor)) {
            console.warn("[Bucket]", "Unable to open popover. Anchor must be an attached DOM-element");
            return;
        }
    }
    (0, preact_1.render)((0, preact_1.h)(FeedbackDialog_1.FeedbackDialog, Object.assign(Object.assign({}, options), { position })), shadowRoot);
    const dialog = shadowRoot.querySelector("dialog");
    if (dialog && !dialog.hasAttribute("open")) {
        dialog[position.type === "MODAL" ? "showModal" : "show"]();
    }
}
exports.openFeedbackForm = openFeedbackForm;
//# sourceMappingURL=index.js.map