"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.arrow = void 0;
const dom_1 = require("@floating-ui/dom");
/**
 * Provides data to position an inner element of the floating element so that it
 * appears centered to the reference element.
 * This wraps the core `arrow` middleware to allow React refs as the element.
 * @see https://floating-ui.com/docs/arrow
 */
const arrow = (options) => {
    function isRef(value) {
        return {}.hasOwnProperty.call(value, "current");
    }
    return {
        name: "arrow",
        options,
        fn(state) {
            const { element, padding } = typeof options === "function" ? options(state) : options;
            if (element && isRef(element)) {
                if (element.current != null) {
                    return (0, dom_1.arrow)({ element: element.current, padding }).fn(state);
                }
                return {};
            }
            else if (element) {
                return (0, dom_1.arrow)({ element, padding }).fn(state);
            }
            return {};
        },
    };
};
exports.arrow = arrow;
//# sourceMappingURL=arrow.js.map