"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useFloating = void 0;
const dom_1 = require("@floating-ui/dom");
const hooks_1 = require("preact/hooks");
const deepEqual_1 = require("./utils/deepEqual");
const getDPR_1 = require("./utils/getDPR");
const roundByDPR_1 = require("./utils/roundByDPR");
const useLatestRef_1 = require("./utils/useLatestRef");
/**
 * Provides data to position a floating element.
 * @see https://floating-ui.com/docs/react
 */
function useFloating(options = {}) {
    const { placement = "bottom", strategy = "absolute", middleware = [], platform, elements: { reference: externalReference, floating: externalFloating } = {}, transform = true, whileElementsMounted, open, } = options;
    const [data, setData] = (0, hooks_1.useState)({
        x: 0,
        y: 0,
        strategy,
        placement,
        middlewareData: {},
        isPositioned: false,
    });
    const [latestMiddleware, setLatestMiddleware] = (0, hooks_1.useState)(middleware);
    if (!(0, deepEqual_1.deepEqual)(latestMiddleware, middleware)) {
        setLatestMiddleware(middleware);
    }
    const [_reference, _setReference] = (0, hooks_1.useState)(null);
    const [_floating, _setFloating] = (0, hooks_1.useState)(null);
    const setReference = (0, hooks_1.useCallback)((node) => {
        if (node != referenceRef.current) {
            referenceRef.current = node;
            _setReference(node);
        }
    }, [_setReference]);
    const setFloating = (0, hooks_1.useCallback)((node) => {
        if (node !== floatingRef.current) {
            floatingRef.current = node;
            _setFloating(node);
        }
    }, [_setFloating]);
    const referenceEl = (externalReference || _reference);
    const floatingEl = externalFloating || _floating;
    const referenceRef = (0, hooks_1.useRef)(null);
    const floatingRef = (0, hooks_1.useRef)(null);
    const dataRef = (0, hooks_1.useRef)(data);
    const whileElementsMountedRef = (0, useLatestRef_1.useLatestRef)(whileElementsMounted);
    const platformRef = (0, useLatestRef_1.useLatestRef)(platform);
    const update = (0, hooks_1.useCallback)(() => {
        if (!referenceRef.current || !floatingRef.current) {
            return;
        }
        const config = {
            placement,
            strategy,
            middleware: latestMiddleware,
        };
        if (platformRef.current) {
            config.platform = platformRef.current;
        }
        /*eslint-disable-next-line @typescript-eslint/no-floating-promises*/
        (0, dom_1.computePosition)(referenceRef.current, floatingRef.current, config).then((positionData) => {
            const fullData = Object.assign(Object.assign({}, positionData), { isPositioned: true });
            if (isMountedRef.current && !(0, deepEqual_1.deepEqual)(dataRef.current, fullData)) {
                dataRef.current = fullData;
                setData(fullData);
            }
        });
    }, [latestMiddleware, placement, strategy, platformRef]);
    (0, hooks_1.useLayoutEffect)(() => {
        if (open === false && dataRef.current.isPositioned) {
            dataRef.current.isPositioned = false;
            setData((positionData) => (Object.assign(Object.assign({}, positionData), { isPositioned: false })));
        }
    }, [open]);
    const isMountedRef = (0, hooks_1.useRef)(false);
    (0, hooks_1.useLayoutEffect)(() => {
        isMountedRef.current = true;
        return () => {
            isMountedRef.current = false;
        };
    }, []);
    (0, hooks_1.useLayoutEffect)(() => {
        if (referenceEl)
            referenceRef.current = referenceEl;
        if (floatingEl)
            floatingRef.current = floatingEl;
        if (referenceEl && floatingEl) {
            if (whileElementsMountedRef.current) {
                return whileElementsMountedRef.current(referenceEl, floatingEl, update);
            }
            else {
                return update();
            }
        }
    }, [referenceEl, floatingEl, update, whileElementsMountedRef]);
    const refs = (0, hooks_1.useMemo)(() => ({
        reference: referenceRef,
        floating: floatingRef,
        setReference,
        setFloating,
    }), [setReference, setFloating]);
    const elements = (0, hooks_1.useMemo)(() => ({ reference: referenceEl, floating: floatingEl }), [referenceEl, floatingEl]);
    const floatingStyles = (0, hooks_1.useMemo)(() => {
        const initialStyles = {
            position: strategy,
            left: 0,
            top: 0,
        };
        if (!elements.floating) {
            return initialStyles;
        }
        const x = (0, roundByDPR_1.roundByDPR)(elements.floating, data.x);
        const y = (0, roundByDPR_1.roundByDPR)(elements.floating, data.y);
        if (transform) {
            return Object.assign(Object.assign(Object.assign({}, initialStyles), { transform: `translate(${x}px, ${y}px)` }), ((0, getDPR_1.getDPR)(elements.floating) >= 1.5 && { willChange: "transform" }));
        }
        return {
            position: strategy,
            left: x,
            top: y,
        };
    }, [strategy, transform, elements.floating, data.x, data.y]);
    return (0, hooks_1.useMemo)(() => (Object.assign(Object.assign({}, data), { update,
        refs,
        elements,
        floatingStyles })), [data, update, refs, elements, floatingStyles]);
}
exports.useFloating = useFloating;
//# sourceMappingURL=useFloating.js.map