"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const is_bundling_for_browser_or_node_1 = require("is-bundling-for-browser-or-node");
const config_1 = require("./config");
const default_feedback_prompt_handler_1 = require("./default-feedback-prompt-handler");
const feedbackLib = __importStar(require("./feedback"));
const flags_fetch_1 = require("./flags-fetch");
const prompt_storage_1 = require("./prompt-storage");
const prompts_1 = require("./prompts");
const sse_1 = require("./sse");
function postRequest(url, body) {
    return __awaiter(this, void 0, void 0, function* () {
        return (0, cross_fetch_1.default)(url, {
            method: "post",
            headers: {
                "Content-Type": "application/json",
                [config_1.SDK_VERSION_HEADER_NAME]: config_1.SDK_VERSION,
            },
            body: JSON.stringify(body),
        });
    });
}
const isMobile = typeof window !== "undefined" && window.innerWidth < 768;
function main() {
    let debug = false;
    let publishableKey = undefined;
    let host = config_1.API_HOST;
    let sseHost = config_1.SSE_REALTIME_HOST;
    let sessionUserId = undefined;
    let persistUser = !is_bundling_for_browser_or_node_1.isForNode;
    let liveSatisfactionActive = false;
    let sseChannel = undefined;
    let liveSatisfactionEnabled = !is_bundling_for_browser_or_node_1.isForNode;
    let feedbackPromptHandler = undefined;
    let feedbackPromptingUserId = undefined;
    let feedbackPosition = undefined;
    let feedbackTranslations = undefined;
    log("Instance created");
    function getUrl() {
        return `${host}/${publishableKey}`;
    }
    function checkKey() {
        if (!publishableKey) {
            err("Publishable key is not set, please call init() first");
        }
    }
    function getSessionUser() {
        if (!sessionUserId) {
            err("User is not set, please call user() first or provide userId as argument");
        }
        return sessionUserId;
    }
    function log(message, ...args) {
        if (debug) {
            console.log("[Bucket]", message, ...args);
        }
    }
    function warn(message, ...args) {
        console.warn("[Bucket]", message, ...args);
    }
    function err(message, ...args) {
        if (debug) {
            console.error("[Bucket]", message, ...args);
        }
        throw new Error(message);
    }
    function resolveUser(userId) {
        if (userId) {
            return userId;
        }
        if (persistUser) {
            return getSessionUser();
        }
        else {
            err("No userId provided and persistUser is disabled");
        }
    }
    /**
     * Initialize the Bucket SDK.
     *
     * Must be called before calling other SDK methods.
     *
     * @param key Your Bucket publishable key
     * @param options
     */
    function init(key, options = {}) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
        reset();
        if (!key) {
            err("Publishable key was not provided");
        }
        publishableKey = key;
        if (options.debug)
            debug = options.debug;
        if (options.host)
            host = options.host;
        if (options.sseHost)
            sseHost = options.sseHost;
        if ((_b = (_a = options.feedback) === null || _a === void 0 ? void 0 : _a.ui) === null || _b === void 0 ? void 0 : _b.position) {
            feedbackPosition = (_d = (_c = options.feedback) === null || _c === void 0 ? void 0 : _c.ui) === null || _d === void 0 ? void 0 : _d.position;
        }
        if ((_f = (_e = options.feedback) === null || _e === void 0 ? void 0 : _e.ui) === null || _f === void 0 ? void 0 : _f.translations) {
            feedbackTranslations = (_h = (_g = options.feedback) === null || _g === void 0 ? void 0 : _g.ui) === null || _h === void 0 ? void 0 : _h.translations;
        }
        if (typeof options.persistUser !== "undefined") {
            persistUser = options.persistUser;
        }
        if (typeof ((_j = options.feedback) === null || _j === void 0 ? void 0 : _j.enableLiveFeedback) !== "undefined") {
            liveSatisfactionEnabled = options.feedback.enableLiveFeedback;
        }
        if (typeof ((_k = options.feedback) === null || _k === void 0 ? void 0 : _k.enableLiveSatisfaction) !== "undefined") {
            liveSatisfactionEnabled = options.feedback.enableLiveSatisfaction;
        }
        if (liveSatisfactionEnabled && is_bundling_for_browser_or_node_1.isForNode) {
            err("Feedback prompting is not supported in Node.js environment");
        }
        if (liveSatisfactionEnabled && !persistUser) {
            err("Feedback prompting is not supported when persistUser is disabled");
        }
        feedbackPromptHandler =
            (_p = (_m = (_l = options.feedback) === null || _l === void 0 ? void 0 : _l.liveFeedbackHandler) !== null && _m !== void 0 ? _m : (_o = options.feedback) === null || _o === void 0 ? void 0 : _o.liveSatisfactionHandler) !== null && _p !== void 0 ? _p : (0, default_feedback_prompt_handler_1.createDefaultFeedbackPromptHandler)((_q = options.feedback) === null || _q === void 0 ? void 0 : _q.ui);
        log(`initialized with key "${publishableKey}" and options`, options);
    }
    /**
     * Identify the current user in Bucket, so they are tracked against each `track` call and receive Live Satisfaction events.
     *
     * @param userId The ID you use to identify this user
     * @param attributes Any attributes you want to attach to the user in Bucket
     * @param context
     */
    function user(userId, attributes, context) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (!userId)
                err("No userId provided");
            if (persistUser) {
                if (sessionUserId && sessionUserId !== userId) {
                    reset();
                }
                sessionUserId = userId;
                if (liveSatisfactionEnabled && !liveSatisfactionActive) {
                    yield initLiveSatisfaction(userId);
                }
            }
            const payload = {
                userId,
                attributes,
                context,
            };
            const res = yield postRequest(`${getUrl()}/user`, payload);
            log(`sent user`, res);
            return res;
        });
    }
    /**
     * Identify the current user's company in Bucket, so it is tracked against each `track` call.
     *
     * @param companyId The ID you use to identify this company
     * @param attributes Any attributes you want to attach to the company in Bucket
     * @param userId The ID you use to identify the current user
     * @param context
     */
    function company(companyId, attributes, userId, context) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (!companyId)
                err("No companyId provided");
            userId = resolveUser(userId);
            const payload = {
                userId,
                companyId,
                context,
            };
            if (attributes)
                payload.attributes = attributes;
            const res = yield postRequest(`${getUrl()}/company`, payload);
            log(`sent company`, res);
            return res;
        });
    }
    /**
     * Track an event in Bucket.
     *
     * @param eventName The name of the event
     * @param attributes Any attributes you want to attach to the event
     * @param userId The ID you use to identify the current user
     * @param companyId The ID you use to identify the current user's company
     * @param context
     */
    function track(eventName, attributes, userId, companyId, context) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (!eventName)
                err("No eventName provided");
            userId = resolveUser(userId);
            const payload = {
                userId,
                event: eventName,
                companyId,
                context,
            };
            if (attributes)
                payload.attributes = attributes;
            const res = yield postRequest(`${getUrl()}/event`, payload);
            log(`sent event`, res);
            return res;
        });
    }
    /**
     * Submit user feedback to Bucket. Must include either `score` or `comment`, or both.
     *
     * @param options
     * @returns
     */
    function feedback({ feedbackId, featureId, question, score, userId, companyId, comment, promptId, promptedQuestion, source, }) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (!featureId)
                err("No featureId provided");
            if (!score && !comment)
                err("Either 'score' or 'comment' must be provided");
            userId = resolveUser(userId);
            const payload = {
                feedbackId,
                userId,
                featureId,
                score,
                companyId,
                comment,
                promptId,
                question,
                promptedQuestion,
                source: source !== null && source !== void 0 ? source : "sdk",
            };
            const res = yield postRequest(`${getUrl()}/feedback`, payload);
            log(`sent feedback`, res);
            return res;
        });
    }
    /**
     * @deprecated Use `initLiveSatisfaction` instead
     */
    const initLiveFeedback = initLiveSatisfaction;
    /**
     * Start receiving Live Satisfaction feedback prompts.
     *
     * This doesn't need to be called unless you set `enableLiveSatisfaction` to false when calling `init`.
     *
     * @param userId The ID you use to identify the user
     */
    function initLiveSatisfaction(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (is_bundling_for_browser_or_node_1.isForNode) {
                err("Feedback prompting is not supported in Node.js environment");
            }
            if (liveSatisfactionActive) {
                err("Feedback prompting already initialized. Use reset() first.");
            }
            if (isMobile) {
                warn("Feedback prompting is not supported on mobile devices");
                return;
            }
            userId = resolveUser(userId);
            const existingAuth = (0, prompt_storage_1.getAuthToken)(userId);
            let channel = existingAuth === null || existingAuth === void 0 ? void 0 : existingAuth.channel;
            // while initializing, consider the channel active
            liveSatisfactionActive = true;
            try {
                if (!channel) {
                    const res = yield postRequest(`${getUrl()}/feedback/prompting-init`, {
                        userId,
                    });
                    log(`feedback prompting status sent`, res);
                    const body = yield res.json();
                    if (!body.success || !body.channel) {
                        log(`feedback prompting not enabled`);
                        return res;
                    }
                    channel = body.channel;
                }
                log(`feedback prompting enabled`, channel);
                sseChannel = (0, sse_1.openAblySSEChannel)(`${getUrl()}/feedback/prompting-auth`, userId, channel, (message) => handleFeedbackPromptRequest(userId, message), { debug, sseHost });
                feedbackPromptingUserId = userId;
                log(`feedback prompting connection established`);
            }
            finally {
                // check that SSE channel has actually been opened, otherwise reset the value
                liveSatisfactionActive = !!sseChannel;
            }
            return channel;
        });
    }
    function handleFeedbackPromptRequest(userId, message) {
        const parsed = (0, prompts_1.parsePromptMessage)(message);
        if (!parsed) {
            err(`invalid feedback prompt message received`, message);
        }
        else {
            if (!(0, prompts_1.processPromptMessage)(userId, parsed, (u, m, cb) => __awaiter(this, void 0, void 0, function* () {
                yield feedbackPromptEvent({
                    promptId: parsed.promptId,
                    featureId: parsed.featureId,
                    promptedQuestion: parsed.question,
                    event: "received",
                    userId,
                });
                yield triggerFeedbackPrompt(u, m, cb);
            }))) {
                log(`feedback prompt not shown, it was either expired or already processed`, message);
            }
        }
    }
    function triggerFeedbackPrompt(userId, message, completionHandler) {
        return __awaiter(this, void 0, void 0, function* () {
            let feedbackId = undefined;
            if (feedbackPromptingUserId !== userId) {
                log(`feedback prompt not shown, received for another user`, userId, message);
                return;
            }
            yield feedbackPromptEvent({
                promptId: message.promptId,
                featureId: message.featureId,
                promptedQuestion: message.question,
                event: "shown",
                userId,
            });
            const replyCallback = (reply) => __awaiter(this, void 0, void 0, function* () {
                if (!reply) {
                    yield feedbackPromptEvent({
                        promptId: message.promptId,
                        featureId: message.featureId,
                        event: "dismissed",
                        userId,
                        promptedQuestion: message.question,
                    });
                    completionHandler();
                    return;
                }
                const response = yield feedback({
                    feedbackId: feedbackId,
                    featureId: message.featureId,
                    userId,
                    companyId: reply.companyId,
                    score: reply.score,
                    comment: reply.comment,
                    promptId: message.promptId,
                    question: reply.question,
                    promptedQuestion: message.question,
                    source: "prompt",
                });
                completionHandler();
                return yield response.json();
            });
            const handlers = {
                reply: replyCallback,
                openFeedbackForm: (options) => {
                    feedbackLib.openFeedbackForm(Object.assign({ key: message.featureId, title: message.question, onScoreSubmit: (data) => __awaiter(this, void 0, void 0, function* () {
                            const res = yield replyCallback(data);
                            feedbackId = res.feedbackId;
                            return { feedbackId: res.feedbackId };
                        }), onSubmit: (data) => __awaiter(this, void 0, void 0, function* () {
                            var _a;
                            yield replyCallback(data);
                            (_a = options.onAfterSubmit) === null || _a === void 0 ? void 0 : _a.call(options, data);
                        }), onDismiss: () => replyCallback(null), position: feedbackPosition, translations: feedbackTranslations }, options));
                },
            };
            feedbackPromptHandler === null || feedbackPromptHandler === void 0 ? void 0 : feedbackPromptHandler(message, handlers);
        });
    }
    function feedbackPromptEvent(args) {
        return __awaiter(this, void 0, void 0, function* () {
            checkKey();
            if (!args.promptId)
                err("No promptId provided");
            if (!args.event)
                err("No event provided");
            const payload = {
                action: args.event,
                featureId: args.featureId,
                promptId: args.promptId,
                userId: args.userId,
                promptedQuestion: args.promptedQuestion,
            };
            const res = yield postRequest(`${getUrl()}/feedback/prompt-events`, payload);
            log(`sent prompt event`, res);
            return res;
        });
    }
    /**
     * Display the Bucket feedback form UI programmatically.
     *
     * This can be used to collect feedback from users in Bucket in cases where Live Satisfaction isn't appropriate.
     *
     * @param options
     */
    function requestFeedback(options) {
        if (is_bundling_for_browser_or_node_1.isForNode) {
            err("requestFeedback can only be called in the browser");
        }
        if (!options.featureId) {
            err("No featureId provided");
        }
        const userId = resolveUser(options.userId);
        // Wait a tick before opening the feedback form,
        // to prevent the same click from closing it.
        setTimeout(() => {
            var _a, _b;
            feedbackLib.openFeedbackForm({
                key: options.featureId,
                title: options.title,
                position: (_a = options.position) !== null && _a !== void 0 ? _a : feedbackPosition,
                translations: (_b = options.translations) !== null && _b !== void 0 ? _b : feedbackTranslations,
                openWithCommentVisible: options.openWithCommentVisible,
                onClose: options.onClose,
                onDismiss: options.onDismiss,
                onScoreSubmit: (data) => __awaiter(this, void 0, void 0, function* () {
                    const res = yield feedback(Object.assign({ featureId: options.featureId, userId, companyId: options.companyId, source: "widget" }, data));
                    const json = yield res.json();
                    return { feedbackId: json.feedbackId };
                }),
                onSubmit: (data) => __awaiter(this, void 0, void 0, function* () {
                    var _c;
                    // Default onSubmit handler
                    yield feedback(Object.assign({ featureId: options.featureId, userId, companyId: options.companyId, source: "widget" }, data));
                    (_c = options.onAfterSubmit) === null || _c === void 0 ? void 0 : _c.call(options, data);
                }),
            });
        }, 1);
    }
    /**
     * Retrieve feature flags given the specified context.
     *
     *
     * @param context The context to evaluate the feature flags against.
     *                This should take the form of `{ user: { id }, company: { id } }`
     *                plus anything additional you want to be able to evaluate flags against.
     *                In the browser, if a `user` call has been made the `user.id` will be
     *                used automatically and merged with anything provided in the `context`
     *                argument.
     * @param fallbackFlags Optional array of flags to use if the request to Bucket fails.
     * @param timeoutMs Optional the timeout in milliseconds for the request to Bucket before using `fallbackFlags`. Default is 5000ms.
     * @param staleWhileRevalidate Optional whether to return potentially stale flags while revalidating the flags in the background. Default is true.
     */
    function getFeatureFlags({ context, fallbackFlags = [], timeoutMs, staleWhileRevalidate = true, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const baseContext = {
                user: { id: sessionUserId },
            };
            const mergedContext = (0, flags_fetch_1.mergeDeep)(baseContext, context);
            const flags = yield (0, flags_fetch_1.getFlags)({
                apiBaseUrl: getUrl(),
                context: mergedContext,
                timeoutMs,
                staleWhileRevalidate,
            });
            if (!flags) {
                warn(`failed to fetch feature flags, using fall-back flags`);
                return fallbackFlags.reduce((acc, flag) => {
                    acc[flag] = true;
                    return acc;
                }, {});
            }
            return Object.fromEntries(Object.keys(flags).map((key) => [key, true]));
        });
    }
    /**
     * Reset the active user and disconnect from Live Satisfaction events.
     */
    function reset() {
        sessionUserId = undefined;
        feedbackPromptingUserId = undefined;
        liveSatisfactionActive = false;
        if (sseChannel) {
            (0, sse_1.closeAblySSEChannel)(sseChannel);
            log(`feedback prompting connection closed`);
            sseChannel = undefined;
        }
    }
    return {
        // lifecycle
        init,
        reset,
        version: config_1.SDK_VERSION,
        // requests
        user,
        company,
        track,
        feedback,
        // feedback prompting
        requestFeedback,
        initLiveSatisfaction,
        initLiveFeedback,
        // feature flags
        getFeatureFlags,
    };
}
exports.default = main;
//# sourceMappingURL=main.js.map