"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forgetAuthToken = exports.getAuthToken = exports.rememberAuthToken = exports.checkPromptMessageCompleted = exports.markPromptMessageCompleted = void 0;
const js_cookie_1 = __importDefault(require("js-cookie"));
const markPromptMessageCompleted = (userId, promptId, expiresAt) => {
    js_cookie_1.default.set(`bucket-prompt-${userId}`, promptId, {
        expires: expiresAt,
        sameSite: "strict",
        secure: true,
    });
};
exports.markPromptMessageCompleted = markPromptMessageCompleted;
const checkPromptMessageCompleted = (userId, promptId) => {
    const id = js_cookie_1.default.get(`bucket-prompt-${userId}`);
    return id === promptId;
};
exports.checkPromptMessageCompleted = checkPromptMessageCompleted;
const rememberAuthToken = (userId, channel, token, expiresAt) => {
    js_cookie_1.default.set(`bucket-token-${userId}`, JSON.stringify({ channel, token }), {
        expires: expiresAt,
        sameSite: "strict",
        secure: true,
    });
};
exports.rememberAuthToken = rememberAuthToken;
const getAuthToken = (userId) => {
    const val = js_cookie_1.default.get(`bucket-token-${userId}`);
    if (!val) {
        return undefined;
    }
    try {
        const { channel, token } = JSON.parse(val);
        if (!(channel === null || channel === void 0 ? void 0 : channel.length) || !(token === null || token === void 0 ? void 0 : token.length)) {
            return undefined;
        }
        return {
            channel,
            token,
        };
    }
    catch (e) {
        return undefined;
    }
};
exports.getAuthToken = getAuthToken;
const forgetAuthToken = (userId) => {
    js_cookie_1.default.remove(`bucket-token-${userId}`);
};
exports.forgetAuthToken = forgetAuthToken;
//# sourceMappingURL=prompt-storage.js.map