"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processPromptMessage = exports.parsePromptMessage = void 0;
const prompt_storage_1 = require("./prompt-storage");
const parsePromptMessage = (message) => {
    if (typeof (message === null || message === void 0 ? void 0 : message.question) !== "string" ||
        !message.question.length ||
        typeof message.showAfter !== "number" ||
        typeof message.showBefore !== "number" ||
        typeof message.promptId !== "string" ||
        !message.promptId.length ||
        typeof message.featureId !== "string" ||
        !message.featureId.length) {
        return undefined;
    }
    else {
        return {
            question: message.question,
            showAfter: new Date(message.showAfter),
            showBefore: new Date(message.showBefore),
            promptId: message.promptId,
            featureId: message.featureId,
        };
    }
};
exports.parsePromptMessage = parsePromptMessage;
const processPromptMessage = (userId, prompt, displayHandler) => {
    const now = new Date();
    const completionHandler = () => {
        (0, prompt_storage_1.markPromptMessageCompleted)(userId, prompt.promptId, prompt.showBefore);
    };
    if ((0, prompt_storage_1.checkPromptMessageCompleted)(userId, prompt.promptId)) {
        return false;
    }
    else if (now > prompt.showBefore) {
        return false;
    }
    else if (now < prompt.showAfter) {
        setTimeout(() => {
            displayHandler(userId, prompt, completionHandler);
        }, prompt.showAfter.getTime() - now.getTime());
        return true;
    }
    else {
        displayHandler(userId, prompt, completionHandler);
        return true;
    }
};
exports.processPromptMessage = processPromptMessage;
//# sourceMappingURL=prompts.js.map